using System.Collections.Generic;

using AntMe.English;


//Insert a slashdot behind AntMe.Player followed by your name (please don't use blanks).
//It could look like this: "AntMe.Player.NickBoesl".
namespace AntMe.Player.gep150
{
    //Using the 'player'-attribute you can specify the name of your ant colony and the 
    //first and last name of the player. In order for the game to find your ant colony you
    //have to name your colony; inserting your name 
    [Player(
        ColonyName = "gep150",
        FirstName = "Valentin",
        LastName = "Kirilov"
        )]

    //The 'caste'-attribute makes it possible to change the properties of your ants. You can make 
    //them faster or give them more energy etc. But be careful: Only '-1', '0', '1' and '2' can be
    //be used and the sum of all properties of one caste has to be '0'!
    //To use this attribute you have to name the caste fist. After that you can return the name of that
    //type by using 'ChooseType'.
    //For further information how to use the ant-types see chapter 6 of the tutorial. 

    [Caste(
        Name = "Killer",
        SpeedModificator = 0,
        LoadModificator = -1,
        AttackModificator = 2,
        EnergyModificator = 1,
        RangeModificator = -1,
        ViewRangeModificator = -1,
        RotationSpeedModificator = 0
        )]

    [Caste(
        Name = "Fighter",
        SpeedModificator = 1,
        LoadModificator = -1,
        AttackModificator = 1,
        EnergyModificator = 0,
        RangeModificator = 0,
        ViewRangeModificator = -1,
        RotationSpeedModificator = 0
        )]

    [Caste(
        Name = "Farmer",
        SpeedModificator = 1,
        LoadModificator = 1,
        AttackModificator = 0,
        EnergyModificator = -1,
        RangeModificator = 0,
        ViewRangeModificator = -1,
        RotationSpeedModificator = 0
        )]

    public class FinalAnts : BaseAnt
    {
        #region Castes

        public override string ChooseType(Dictionary<string, int> typeCount)
        {
            if (typeCount["Killer"] < 13)
                return "Killer";
            else if (typeCount["Farmer"] < 27)
                return "Farmer";
            else 
                return "Fighter";
        }

        #endregion

        #region Movement

        public override void Waits() 
        {
            GoAhead();
        }

        public override void BecomesTired() 
        { 
        }

        #endregion

        #region Food

        public override void Spots(Sugar sugar)
        {
            int distance, direction; 
            distance = Coordinate.GetDistanceBetween(this, sugar); 
            direction = Coordinate.GetDegreesBetween(this, sugar); 
            MakeMark(direction, distance);

            if (Caste == "Farmer")
            {            
                if (CurrentLoad == 0) 
                {
                    if (!(Target is Bug)) 
                    {
                        GoToTarget(sugar); 
                    }
                }
            }
        }

        public override void Spots(Fruit fruit) 
        {
            int distance, direction;
            distance = Coordinate.GetDistanceBetween(this, fruit);
            direction = Coordinate.GetDegreesBetween(this, fruit);
            MakeMark(direction, distance);

            if (Caste != "Killer")
            {
                if (CurrentLoad == 0 && NeedsCarrier(fruit))
                {
                    if (!(Target is Bug))
                    {
                        if (Caste == "Farmer")
                        {
                            GoToTarget(fruit);    
                        }
                        else if (Caste == "Fighter" && ForeignAntsInViewrange == 0)
                        {
                            GoToTarget(fruit);    
                        }
                    }
                }
            }
        }

        public override void TargetReached(Sugar sugar) 
        {
            Take(sugar);
            GoBackToAnthill();
        }

        public override void TargetReached(Fruit fruit) 
        {
            if (NeedsCarrier(fruit))
            {
                Take(fruit);
                GoBackToAnthill();   
            }
        }

        #endregion

        #region Communication

        public override void SmellsFriend(Marker marker) 
        {
            if (Target == null) 
            {
                TurnToDirection(marker.Information); 
                GoAhead(); 
            }
        }

        public override void SpotsFriend(Ant ant) 
        { 
        }

        public override void SpotsPartner(Ant ant) 
        { 
        }

        #endregion

        #region Fight

        public override void SpotsEnemy(Bug bug) 
        {
            int distance, direction;
            distance = Coordinate.GetDistanceBetween(this, bug);
            direction = Coordinate.GetDegreesBetween(this, bug);
            MakeMark(direction, distance);
            
            Drop();
            if (FriendlyAntsInViewrange * MaximumEnergy > bug.CurrentEnergy)
            {
                Attack(bug);
            }
            else
            {
                if (CurrentLoad == 0 || Target == null)
                {
                    GoAwayFromTarget(bug);
                }
            }   
        }

        public override void SpotsEnemy(Ant ant) 
        {
            if (Caste == "Killer" && !(Target is Bug))
            {
                Attack(ant);
            }
            else if (Caste == "Fighter" && !(Target is Bug))
            {
                Drop();
                Attack(ant);
            }
            else if (Target == null && FriendlyAntsInViewrange * MaximumEnergy > ant.CurrentEnergy) 
            { 
                Attack(ant); 
            }
        }

        public override void UnderAttack(Bug bug) 
        {
            if (Caste == "Killer" || Caste == "Fighter")
            {
                Attack(bug);
            }
            else if (CurrentEnergy < MaximumEnergy / 2)
            {
                GoBackToAnthill();
            }
        }

        public override void UnderAttack(Ant ant) 
        {
            if (Caste == "Killer" || Caste == "Fighter")
            {
                Drop();
                Attack(ant);
            }
            else if (FriendlyAntsInViewrange * MaximumEnergy > ant.CurrentEnergy)
            {
                Drop();
                Attack(ant);
            }
        }

        #endregion

        #region Misc

        public override void HasDied(KindOfDeath kindOfDeath) 
        {
        }

        public override void Tick() 
        {
            if (Target is Anthill && CurrentLoad > 0 && CarringFruit == null) 
            { 
                MakeMark(Direction + 180); 
            }
        }

        #endregion
    }
}